package org.openstreetmap.josm.plugins.lanetool.view;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import org.openstreetmap.josm.plugins.lanetool.data.Lane;
import org.openstreetmap.josm.plugins.lanetool.data.RelationLane;

/**
 * context menu for clicks on a lane
 */
class LanePopupMenu extends JPopupMenu {
		
	/** TODO: proper serialVersionUID */
	private static final long serialVersionUID = 1L;

	/**
	 * 
	 * @param lane  lane affected by the user's choice in this menu
	 */
	public LanePopupMenu(Lane lane) {
		
		String[] laneValues = {"cycleway", "footway", "vehicle_lane", "bus_lane"};
	
		{
			JMenuItem deleteItem = new JMenuItem("delete");
			deleteItem.addActionListener(new DeleteLaneListener(lane));
			this.add(deleteItem);
						
		}
		
		{
			JMenu changeTypeMenu = new JMenu("lane type");
			this.add(changeTypeMenu);

			for (String laneValue : laneValues) {			
				JMenuItem typeItem = new JMenuItem(laneValue);
				typeItem.addActionListener(new SetTypeListener(lane, laneValue));
				changeTypeMenu.add(typeItem);			
			}
		}
		
		{
			JMenu addLeftMenu = new JMenu("add to left");
			this.add(addLeftMenu);

			JMenuItem editAddLeftItem = new JMenuItem("<<manual>>");
			editAddLeftItem.addActionListener(new InsertSiblingLaneListener(lane, null, false));
			addLeftMenu.add(editAddLeftItem);

			for (String presetLabel : laneValues) {			
				JMenuItem leftPresetItem = new JMenuItem(presetLabel);
				leftPresetItem.addActionListener(new InsertSiblingLaneListener(lane, presetLabel, false));
				addLeftMenu.add(leftPresetItem);			
			}
		}
		
		{
			JMenu addRightMenu = new JMenu("add to right");
			this.add(addRightMenu);

			JMenuItem editAddRightItem = new JMenuItem("<<manual>>");
			editAddRightItem.addActionListener(new InsertSiblingLaneListener(lane, null, true));
			addRightMenu.add(editAddRightItem);

			for (String presetLabel : laneValues) {			
				JMenuItem rightPresetItem = new JMenuItem(presetLabel);
				rightPresetItem.addActionListener(new InsertSiblingLaneListener(lane, presetLabel, true));
				addRightMenu.add(rightPresetItem);			
			}
		}
		
		if (lane instanceof RelationLane) {
			
			JMenuItem transformToWayItem = new JMenuItem("to way");
			transformToWayItem.addActionListener(new TransformToWayListener((RelationLane)lane));
			this.add(transformToWayItem);
			
			JMenuItem editItem = new JMenuItem("edit manually");
			editItem.addActionListener(new RelationEditListener((RelationLane)lane));
			this.add(editItem);

		}
		
	}
	
	private static class SetTypeListener implements ActionListener {

		private Lane lane;
		private String laneType;
		
		public SetTypeListener(Lane lane, String laneType) {
			this.lane = lane;
			this.laneType = laneType;
		}

		public void actionPerformed(ActionEvent arg0) {
			lane.setLaneType(laneType);
		}
		
	}
	
	private static class RelationEditListener implements ActionListener {

		private RelationLane lane;
				
		public RelationEditListener(RelationLane lane) {
			this.lane = lane;
		}

		public void actionPerformed(ActionEvent arg0) {
			lane.editRelation();
		}
		
	}
		
	private static class InsertSiblingLaneListener implements ActionListener {

		private Lane lane;
		private String laneType;
		private boolean toRight;
		
		/**
		 * 
		 * @param lane  
		 * @param laneType  type for the sibling lane; may be null to indicate manual relation editing
		 */
		public InsertSiblingLaneListener(Lane lane, String laneType, boolean toRight) {
			this.lane = lane;
			this.laneType = laneType;
			this.toRight = toRight;
		}

		public void actionPerformed(ActionEvent arg0) {
			
			RelationLane sibling = lane.addSibling(laneType, toRight);
			
			if (laneType == null) {
				sibling.editRelation();				
			}
		}
		
	}
	
	private static class TransformToWayListener implements ActionListener {

		private RelationLane lane;
				
		public TransformToWayListener(RelationLane lane) {
			this.lane = lane;
		}

		public void actionPerformed(ActionEvent arg0) {
			lane.transformToWay();
		}
		
	}
	
	private static class DeleteLaneListener implements ActionListener {

		private Lane lane;
				
		public DeleteLaneListener(Lane lane) {
			this.lane = lane;
		}

		public void actionPerformed(ActionEvent arg0) {
			lane.delete();
		}
		
	}
	
}
