package org.openstreetmap.josm.plugins.lanetool.view;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.openstreetmap.josm.gui.dialogs.ToggleDialog;
import org.openstreetmap.josm.plugins.lanetool.data.LanesData;
import org.openstreetmap.josm.tools.Shortcut;

/**
 * Dialog for lane editing. 
 * Observes a LaneData object, displays its current state and modifies it on user actions.
 */
public class LanesDialog extends ToggleDialog implements Observer {
	
	/** TODO: serial version UID */
	private static final long serialVersionUID = -2713827155304960761L;
		
	private static final int HEIGHT = 150;
	
	private JPanel mainPanel;
	private LanesPanel visualPanel;
	
	private JLabel debugLabel;
	
	public LanesDialog(LanesData lanesData) {
				
		super("Lane Tool", "lanetool", "Open the lane tool for editing lane relations.", (Shortcut)null, HEIGHT);
		
		lanesData.addObserver(this);
		
		//mainPanel = new JPanel();
				
		visualPanel = new LanesPanel(lanesData);		
		this.add(BorderLayout.CENTER , visualPanel);
		
		JPanel debugPanel = new JPanel();
		debugLabel = new JLabel("PROTOTYPE -- ");
		debugPanel.add(debugLabel);
		JButton infoButton = new JButton("info");
		infoButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				displayInfo(mainPanel);
			}
		});
		debugPanel.add(infoButton);
		this.add(BorderLayout.SOUTH, debugPanel);
		
		//this.add(mainPanel);
				
		updateView();
		
	}

	public void update(Observable arg0, Object arg1) {
		
		updateView();
		
	}
	
	protected void updateView() {
		
		this.repaint();
		visualPanel.repaint();
		
	}
	
	private static void displayInfo(Component parentComponent) {
		JOptionPane.showMessageDialog(parentComponent, 
			"This plugin creates lanes using a concept described here:\n"
			+ "http://wiki.openstreetmap.org/wiki/Proposed_features/lane_and_lane_group \n"
			+ "The concept is still under discussion.");
	}

}
